/*------------------------------------------------------------------------
 |  FILE            SmarterChecksum.java
 |  PART OF         WebWatcher - A web-page update monitor
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 |            class by Daniel Lemire <Daniel.Lemire@Videotron.ca>
 +----------------------------------------------------------------------*/

import java.io.*;
import java.security.*;


public class SmarterChecksum extends Checksum{

  public SmarterChecksum() {
    super();
  }



  /*
  * essentially the same thing as calling
  * the constructor with a String.
  */
  public void setChecksum(String s) {
        if (s.equals(notInitialized))
            checksum = null;
        else
            checksum = stringToBytes(s);
  }

  /***
   * apply some preprocessing before computing
   * checksum
   */
  /*public void calculate(String s) {
        checksum = digest(convertHTMLtoText(s));
  }*/
  public void calculate(StringBuffer sb) {
//        System.out.println("Calculating checksum...");
        convertHTMLtoText(sb);
        checksum = digest(sb.toString());
  }

  /*
  * First, it checks whether this is HTML, if so
  * it will try to cut out everything outside the <BODY></BODY>.
  * It is based on a guess because HTML, especially bad HTML,
  * can be quite a bit difficult to parse. However, we are
  * making conservative choices.
  * Then, it will remove all tags... leaving only text.
  *
  * Now, that's not entirely accurate to measure the information
  * content of a page. For one thing, images can contain information
  * and so can JavaScript. However, most of the time, a
  * "significative" update will amount to changing the actual
  * text of a web page. We should also remove a lot of white space,
  * which we don't...
  */
  public static void convertHTMLtoText(StringBuffer sb) {
//      System.out.println("Converting to text");
    // first move to lower-case to simplify parsing
    String s = sb.toString();
    String lowercase = s.toLowerCase();
    int EarliestPossibleBeginning = lowercase.indexOf("<html");
    EarliestPossibleBeginning = lowercase.indexOf(">",EarliestPossibleBeginning);
    if (EarliestPossibleBeginning < 0)
      return;// that's not HTML
    int EarliestPossibleBodyBeginning = lowercase.indexOf("<body",EarliestPossibleBeginning) ;
    if (EarliestPossibleBodyBeginning > 0) {
      // we close the bracket
      EarliestPossibleBodyBeginning = lowercase.indexOf(">",EarliestPossibleBodyBeginning);
      if(EarliestPossibleBodyBeginning > 0) {
        // check the the bracket does close!
        EarliestPossibleBodyBeginning++;// right after the end of the body tag
        // ok, now, there is a body tag and we found were it ended...
        EarliestPossibleBeginning = EarliestPossibleBodyBeginning; // there is <body>, assuming beginning there
      }
    }
    int LatestPossibleEnding  = lowercase.lastIndexOf("</body>");
    int LatestPossibleHtmlEnding  = lowercase.lastIndexOf("</html>");

    if (LatestPossibleEnding < 0) {
      if(LatestPossibleHtmlEnding < 0)
        LatestPossibleEnding = lowercase.length();// no end found! just include everything!
      else
        LatestPossibleEnding = LatestPossibleHtmlEnding;// no </body> by </html>... assuming </body>
    } else {
      if ((LatestPossibleHtmlEnding > 0) && (LatestPossibleHtmlEnding < LatestPossibleEnding))
        LatestPossibleEnding = LatestPossibleHtmlEnding; // HTML ends before body! stop with the HTML
    }
    lowercase = null; // allow release of memory as early as possible
    //
    final int EndDelete = s.length() - LatestPossibleEnding;
    sb.delete(0,EarliestPossibleBeginning);
    sb.delete(sb.length() - EndDelete,sb.length());
    int BeginningTag = 0;
    for(int counter = 0; counter < sb.length(); ++counter) {
      switch(sb.charAt(counter)) {
        case  '>':
          if (BeginningTag >= 0) {
            sb.delete(BeginningTag,counter + 1);
            counter -= counter - BeginningTag + 1;
          }
          BeginningTag = -1;
          break;
        case '<':
          BeginningTag = counter;
          break;
      }
    }

    // Now, we are going to trim empty lines and each line separatly
    String line,trimmedline;
    for(int k = 0; k < sb.length(); ++k) {
      int linelength = 0;
      while((sb.charAt(k + linelength) != '\n') && (k + linelength < sb.length() - 1))
        ++linelength;
      line = sb.substring(k,k+linelength);
      trimmedline = line.trim();
      if(trimmedline.length() == 0) {
        sb.delete(k,k+linelength+1);
        --k;
      } else {
        sb.replace(k,trimmedline.length()+k,trimmedline);
        sb.setCharAt(trimmedline.length()+k,'\n');
        if (trimmedline.length()+k+1 <= k+linelength)
          sb.delete(trimmedline.length()+k+1,k+linelength);
        k+=trimmedline.length();
      }

    }

  }
}
